@@ -1,50 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <qjsonobject.h>
 #include <qjsonvalue.h>
 #include <qjsonarray.h>
+#include <qjsondocument.h>
 #include <qstringlist.h>
 #include <qdebug.h>
 #include <qvariant.h>
-#include "qjson_p.h"
+#include <qcbormap.h>
+#include <qmap.h>
+#include <qhash.h>
+
+#include <private/qcborvalue_p.h>
 #include "qjsonwriter_p.h"
+#include "qjson_p.h"
+
+#include <algorithm>
 
 QT_BEGIN_NAMESPACE
 
@@ -100,7 +72,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \typedef QJsonObject::size_type
 
-    Typedef for int. Provided for STL compatibility.
+    Typedef for qsizetype. Provided for STL compatibility.
 */
 
 
@@ -109,10 +81,7 @@ QT_BEGIN_NAMESPACE
 
     \sa isEmpty()
  */
-QJsonObject::QJsonObject()
-    : d(0), o(0)
-{
-}
+QJsonObject::QJsonObject() = default;
 
 /*!
     \fn QJsonObject::QJsonObject(std::initializer_list<QPair<QString, QJsonValue> > args)
@@ -131,35 +100,20 @@ QJsonObject::QJsonObject()
 /*!
     \internal
  */
-QJsonObject::QJsonObject(QJsonPrivate::Data *data, QJsonPrivate::Object *object)
-    : d(data), o(object)
+QJsonObject::QJsonObject(QCborContainerPrivate *object)
+    : o(object)
 {
-    Q_ASSERT(d);
-    Q_ASSERT(o);
-    d->ref.ref();
-}
-
-/*!
-    This method replaces part of the QJsonObject(std::initializer_list<QPair<QString, QJsonValue>> args) body.
-    The constructor needs to be inline, but we do not want to leak implementation details
-    of this class.
-    \note this method is called for an uninitialized object
-    \internal
- */
-
-void QJsonObject::initialize()
-{
-    d = 0;
-    o = 0;
 }
 
 /*!
     Destroys the object.
  */
-QJsonObject::~QJsonObject()
+QJsonObject::~QJsonObject() = default;
+
+QJsonObject::QJsonObject(std::initializer_list<QPair<QString, QJsonValue> > args)
 {
-    if (d && !d->ref.deref())
-        delete d;
+    for (const auto &arg : args)
+        insert(arg.first, arg.second);
 }
 
 /*!
@@ -170,10 +124,13 @@ QJsonObject::~QJsonObject()
  */
 QJsonObject::QJsonObject(const QJsonObject &other)
 {
-    d = other.d;
     o = other.o;
-    if (d)
-        d->ref.ref();
+}
+
+QJsonObject::QJsonObject(QJsonObject &&other) noexcept
+    : o(other.o)
+{
+    other.o = nullptr;
 }
 
 /*!
@@ -181,15 +138,7 @@ QJsonObject::QJsonObject(const QJsonObject &other)
  */
 QJsonObject &QJsonObject::operator =(const QJsonObject &other)
 {
-    if (d != other.d) {
-        if (d && !d->ref.deref())
-            delete d;
-        d = other.d;
-        if (d)
-            d->ref.ref();
-    }
     o = other.o;
-
     return *this;
 }
 
@@ -221,59 +170,14 @@ QJsonObject &QJsonObject::operator =(const QJsonObject &other)
     The keys in \a map will be used as the keys in the JSON object,
     and the QVariant values will be converted to JSON values.
 
+    \note Conversion from \l QVariant is not completely lossless. Please see
+    the documentation in QJsonValue::fromVariant() for more information.
+
     \sa fromVariantHash(), toVariantMap(), QJsonValue::fromVariant()
  */
 QJsonObject QJsonObject::fromVariantMap(const QVariantMap &map)
 {
-    QJsonObject object;
-    if (map.isEmpty())
-        return object;
-
-    object.detach2(1024);
-
-    QVector<QJsonPrivate::offset> offsets;
-    QJsonPrivate::offset currentOffset;
-    currentOffset = sizeof(QJsonPrivate::Base);
-
-    // the map is already sorted, so we can simply append one entry after the other and
-    // write the offset table at the end
-    for (QVariantMap::const_iterator it = map.constBegin(); it != map.constEnd(); ++it) {
-        QString key = it.key();
-        QJsonValue val = QJsonValue::fromVariant(it.value());
-
-        bool latinOrIntValue;
-        int valueSize = QJsonPrivate::Value::requiredStorage(val, &latinOrIntValue);
-
-        bool latinKey = QJsonPrivate::useCompressed(key);
-        int valueOffset = sizeof(QJsonPrivate::Entry) + QJsonPrivate::qStringSize(key, latinKey);
-        int requiredSize = valueOffset + valueSize;
-
-        if (!object.detach2(requiredSize + sizeof(QJsonPrivate::offset))) // offset for the new index entry
-            return QJsonObject();
-
-        QJsonPrivate::Entry *e = reinterpret_cast<QJsonPrivate::Entry *>(reinterpret_cast<char *>(object.o) + currentOffset);
-        e->value.type = val.t;
-        e->value.latinKey = latinKey;
-        e->value.latinOrIntValue = latinOrIntValue;
-        e->value.value = QJsonPrivate::Value::valueToStore(val, (char *)e - (char *)object.o + valueOffset);
-        QJsonPrivate::copyString((char *)(e + 1), key, latinKey);
-        if (valueSize)
-            QJsonPrivate::Value::copyData(val, (char *)e + valueOffset, latinOrIntValue);
-
-        offsets << currentOffset;
-        currentOffset += requiredSize;
-        object.o->size = currentOffset;
-    }
-
-    // write table
-    object.o->tableOffset = currentOffset;
-    if (!object.detach2(sizeof(QJsonPrivate::offset)*offsets.size()))
-        return QJsonObject();
-    memcpy(object.o->table(), offsets.constData(), offsets.size()*sizeof(uint));
-    object.o->length = offsets.size();
-    object.o->size = currentOffset + sizeof(QJsonPrivate::offset)*offsets.size();
-
-    return object;
+    return QJsonPrivate::Variant::toJsonObject(map);
 }
 
 /*!
@@ -285,14 +189,7 @@ QJsonObject QJsonObject::fromVariantMap(const QVariantMap &map)
  */
 QVariantMap QJsonObject::toVariantMap() const
 {
-    QVariantMap map;
-    if (o) {
-        for (uint i = 0; i < o->length; ++i) {
-            QJsonPrivate::Entry *e = o->entryAt(i);
-            map.insert(e->key(), QJsonValue(d, o, e->value).toVariant());
-        }
-    }
-    return map;
+    return QCborMap::fromJsonObject(*this).toVariantMap();
 }
 
 /*!
@@ -302,6 +199,9 @@ QVariantMap QJsonObject::toVariantMap() const
     The keys in \a hash will be used as the keys in the JSON object,
     and the QVariant values will be converted to JSON values.
 
+    \note Conversion from \l QVariant is not completely lossless. Please see
+    the documentation in QJsonValue::fromVariant() for more information.
+
     \sa fromVariantMap(), toVariantHash(), QJsonValue::fromVariant()
  */
 QJsonObject QJsonObject::fromVariantHash(const QVariantHash &hash)
@@ -324,15 +224,7 @@ QJsonObject QJsonObject::fromVariantHash(const QVariantHash &hash)
  */
 QVariantHash QJsonObject::toVariantHash() const
 {
-    QVariantHash hash;
-    if (o) {
-        hash.reserve(o->length);
-        for (uint i = 0; i < o->length; ++i) {
-            QJsonPrivate::Entry *e = o->entryAt(i);
-            hash.insert(e->key(), QJsonValue(d, o, e->value).toVariant());
-        }
-    }
-    return hash;
+    return QCborMap::fromJsonObject(*this).toVariantHash();
 }
 
 /*!
@@ -344,11 +236,9 @@ QStringList QJsonObject::keys() const
 {
     QStringList keys;
     if (o) {
-        keys.reserve(o->length);
-        for (uint i = 0; i < o->length; ++i) {
-            QJsonPrivate::Entry *e = o->entryAt(i);
-            keys.append(e->key());
-        }
+        keys.reserve(o->elements.length() / 2);
+        for (qsizetype i = 0, end = o->elements.length(); i < end; i += 2)
+            keys.append(o->stringAt(i));
     }
     return keys;
 }
@@ -356,12 +246,9 @@ QStringList QJsonObject::keys() const
 /*!
     Returns the number of (key, value) pairs stored in the object.
  */
-int QJsonObject::size() const
+qsizetype QJsonObject::size() const
 {
-    if (!d)
-        return 0;
-
-    return o->length;
+    return o ? o->elements.length() / 2 : 0;
 }
 
 /*!
@@ -371,10 +258,24 @@ int QJsonObject::size() const
  */
 bool QJsonObject::isEmpty() const
 {
-    if (!d)
-        return true;
+    return !o || o->elements.isEmpty();
+}
 
-    return !o->length;
+template<typename String>
+static qsizetype indexOf(const QExplicitlySharedDataPointer<QCborContainerPrivate> &o,
+                         String key, bool *keyExists)
+{
+    const auto begin = QJsonPrivate::ConstKeyIterator(o->elements.constBegin());
+    const auto end = QJsonPrivate::ConstKeyIterator(o->elements.constEnd());
+
+    const auto it = std::lower_bound(
+                begin, end, key,
+                [&](const QJsonPrivate::ConstKeyIterator::value_type &e, const String &key) {
+        return o->stringCompareElement(e.key(), key) < 0;
+    });
+
+    *keyExists = (it != end) && o->stringEqualsElement((*it).key(), key);
+    return it.it - begin.it;
 }
 
 /*!
@@ -386,30 +287,41 @@ bool QJsonObject::isEmpty() const
  */
 QJsonValue QJsonObject::value(const QString &key) const
 {
-    if (!d)
-        return QJsonValue(QJsonValue::Undefined);
+    return value(QStringView(key));
+}
 
-    bool keyExists;
-    int i = o->indexOf(key, &keyExists);
-    if (!keyExists)
-        return QJsonValue(QJsonValue::Undefined);
-    return QJsonValue(d, o, o->entryAt(i)->value);
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonValue QJsonObject::value(QStringView key) const
+{
+    return valueImpl(key);
 }
 
 /*!
     \overload
     \since 5.7
 */
-QJsonValue QJsonObject::value(QLatin1String key) const
+QJsonValue QJsonObject::value(QLatin1StringView key) const
 {
-    if (!d)
+    return valueImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+QJsonValue QJsonObject::valueImpl(T key) const
+{
+    if (!o)
         return QJsonValue(QJsonValue::Undefined);
 
     bool keyExists;
-    int i = o->indexOf(key, &keyExists);
+    auto i = indexOf(o, key, &keyExists);
     if (!keyExists)
         return QJsonValue(QJsonValue::Undefined);
-    return QJsonValue(d, o, o->entryAt(i)->value);
+    return QJsonPrivate::Value::fromTrustedCbor(o->valueAt(i + 1));
 }
 
 /*!
@@ -423,18 +335,27 @@ QJsonValue QJsonObject::value(QLatin1String key) const
  */
 QJsonValue QJsonObject::operator [](const QString &key) const
 {
-    return value(key);
+    return (*this)[QStringView(key)];
 }
 
 /*!
-    \fn QJsonValue QJsonObject::operator [](QLatin1String key) const
+    \fn QJsonValue QJsonObject::operator [](QStringView key) const
+
+    \overload
+    \since 5.14
+*/
+
+/*!
+    \fn QJsonValue QJsonObject::operator [](QLatin1StringView key) const
 
     \overload
     \since 5.7
 */
 
 /*!
-    Returns a reference to the value for \a key.
+    Returns a reference to the value for \a key. If there is no value with key
+    \a key in the object, one is created with a QJsonValue::Null value and then
+    returned.
 
     The return value is of type QJsonValueRef, a helper class for QJsonArray
     and QJsonObject. When you get an object of type QJsonValueRef, you can
@@ -446,24 +367,45 @@ QJsonValue QJsonObject::operator [](const QString &key) const
  */
 QJsonValueRef QJsonObject::operator [](const QString &key)
 {
-    // ### somewhat inefficient, as we lookup the key twice if it doesn't yet exist
-    bool keyExists = false;
-    int index = o ? o->indexOf(key, &keyExists) : -1;
-    if (!keyExists) {
-        iterator i = insert(key, QJsonValue());
-        index = i.i;
-    }
-    return QJsonValueRef(this, index);
+    return (*this)[QStringView(key)];
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonValueRef QJsonObject::operator [](QStringView key)
+{
+    return atImpl(key);
 }
 
 /*!
     \overload
     \since 5.7
 */
-QJsonValueRef QJsonObject::operator [](QLatin1String key)
+QJsonValueRef QJsonObject::operator [](QLatin1StringView key)
 {
-    // ### optimize me
-    return operator[](QString(key));
+    return atImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+QJsonValueRef QJsonObject::atImpl(T key)
+{
+    if (!o)
+        o = new QCborContainerPrivate;
+
+    bool keyExists = false;
+    auto index = indexOf(o, key, &keyExists);
+    if (!keyExists) {
+        detach(o->elements.length() / 2 + 1);
+        o->insertAt(index, key);
+        o->insertAt(index + 1, QCborValue::fromJsonValue(QJsonValue()));
+    }
+    // detaching will happen if and when this QJsonValueRef is assigned to
+    return QJsonValueRef(this, index / 2);
 }
 
 /*!
@@ -481,47 +423,60 @@ QJsonValueRef QJsonObject::operator [](QLatin1String key)
  */
 QJsonObject::iterator QJsonObject::insert(const QString &key, const QJsonValue &value)
 {
-    if (value.t == QJsonValue::Undefined) {
+    return insert(QStringView(key), value);
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonObject::iterator QJsonObject::insert(QStringView key, const QJsonValue &value)
+{
+    return insertImpl(key, value);
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonObject::iterator QJsonObject::insert(QLatin1StringView key, const QJsonValue &value)
+{
+    return insertImpl(key, value);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+QJsonObject::iterator QJsonObject::insertImpl(T key, const QJsonValue &value)
+{
+    if (value.type() == QJsonValue::Undefined) {
         remove(key);
         return end();
     }
-    QJsonValue val = value;
-
-    bool latinOrIntValue;
-    int valueSize = QJsonPrivate::Value::requiredStorage(val, &latinOrIntValue);
-
-    bool latinKey = QJsonPrivate::useCompressed(key);
-    int valueOffset = sizeof(QJsonPrivate::Entry) + QJsonPrivate::qStringSize(key, latinKey);
-    int requiredSize = valueOffset + valueSize;
-
-    if (!detach2(requiredSize + sizeof(QJsonPrivate::offset))) // offset for the new index entry
-        return iterator();
-
-    if (!o->length)
-        o->tableOffset = sizeof(QJsonPrivate::Object);
-
     bool keyExists = false;
-    int pos = o->indexOf(key, &keyExists);
-    if (keyExists)
-        ++d->compactionCounter;
+    auto pos = o ? indexOf(o, key, &keyExists) : 0;
+    return insertAt(pos, key, value, keyExists);
+}
 
-    uint off = o->reserveSpace(requiredSize, pos, 1, keyExists);
-    if (!off)
-        return end();
+/*!
+    \internal
+ */
+template <typename T>
+QJsonObject::iterator QJsonObject::insertAt(qsizetype pos, T key, const QJsonValue &value, bool keyExists)
+{
+    if (o)
+        detach(o->elements.length() / 2 + (keyExists ? 0 : 1));
+    else
+        o = new QCborContainerPrivate;
 
-    QJsonPrivate::Entry *e = o->entryAt(pos);
-    e->value.type = val.t;
-    e->value.latinKey = latinKey;
-    e->value.latinOrIntValue = latinOrIntValue;
-    e->value.value = QJsonPrivate::Value::valueToStore(val, (char *)e - (char *)o + valueOffset);
-    QJsonPrivate::copyString((char *)(e + 1), key, latinKey);
-    if (valueSize)
-        QJsonPrivate::Value::copyData(val, (char *)e + valueOffset, latinOrIntValue);
-
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(o->length) / 2u)
-        compact();
-
-    return iterator(this, pos);
+    if (keyExists) {
+        o->replaceAt(pos + 1, QCborValue::fromJsonValue(value));
+    } else {
+        o->insertAt(pos, key);
+        o->insertAt(pos + 1, QCborValue::fromJsonValue(value));
+    }
+    return {this, pos / 2};
 }
 
 /*!
@@ -531,19 +486,42 @@ QJsonObject::iterator QJsonObject::insert(const QString &key, const QJsonValue &
  */
 void QJsonObject::remove(const QString &key)
 {
-    if (!d)
+    remove(QStringView(key));
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+void QJsonObject::remove(QStringView key)
+{
+    removeImpl(key);
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+void QJsonObject::remove(QLatin1StringView key)
+{
+    removeImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+void QJsonObject::removeImpl(T key)
+{
+    if (!o)
         return;
 
     bool keyExists;
-    int index = o->indexOf(key, &keyExists);
+    auto index = indexOf(o, key, &keyExists);
     if (!keyExists)
         return;
 
-    detach2();
-    o->removeItems(index, 1);
-    ++d->compactionCounter;
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(o->length) / 2u)
-        compact();
+    removeAt(index);
 }
 
 /*!
@@ -556,22 +534,45 @@ void QJsonObject::remove(const QString &key)
     \sa insert(), remove(), QJsonValue
  */
 QJsonValue QJsonObject::take(const QString &key)
+{
+    return take(QStringView(key));
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonValue QJsonObject::take(QStringView key)
+{
+    return takeImpl(key);
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonValue QJsonObject::take(QLatin1StringView key)
+{
+    return takeImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+QJsonValue QJsonObject::takeImpl(T key)
 {
     if (!o)
         return QJsonValue(QJsonValue::Undefined);
 
     bool keyExists;
-    int index = o->indexOf(key, &keyExists);
+    auto index = indexOf(o, key, &keyExists);
     if (!keyExists)
         return QJsonValue(QJsonValue::Undefined);
 
-    QJsonValue v(d, o, o->entryAt(index)->value);
-    detach2();
-    o->removeItems(index, 1);
-    ++d->compactionCounter;
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(o->length) / 2u)
-        compact();
-
+    detach();
+    const QJsonValue v = QJsonPrivate::Value::fromTrustedCbor(o->extractAt(index + 1));
+    removeAt(index);
     return v;
 }
 
@@ -582,25 +583,38 @@ QJsonValue QJsonObject::take(const QString &key)
  */
 bool QJsonObject::contains(const QString &key) const
 {
-    if (!o)
-        return false;
+    return contains(QStringView(key));
+}
 
-    bool keyExists;
-    o->indexOf(key, &keyExists);
-    return keyExists;
+/*!
+    \overload
+    \since 5.14
+*/
+bool QJsonObject::contains(QStringView key) const
+{
+    return containsImpl(key);
 }
 
 /*!
     \overload
     \since 5.7
 */
-bool QJsonObject::contains(QLatin1String key) const
+bool QJsonObject::contains(QLatin1StringView key) const
+{
+    return containsImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+bool QJsonObject::containsImpl(T key) const
 {
     if (!o)
         return false;
 
     bool keyExists;
-    o->indexOf(key, &keyExists);
+    indexOf(o, key, &keyExists);
     return keyExists;
 }
 
@@ -613,16 +627,14 @@ bool QJsonObject::operator==(const QJsonObject &other) const
         return true;
 
     if (!o)
-        return !other.o->length;
+        return !other.o->elements.length();
     if (!other.o)
-        return !o->length;
-    if (o->length != other.o->length)
+        return !o->elements.length();
+    if (o->elements.length() != other.o->elements.length())
         return false;
 
-    for (uint i = 0; i < o->length; ++i) {
-        QJsonPrivate::Entry *e = o->entryAt(i);
-        QJsonValue v(d, o, e->value);
-        if (other.value(e->key()) != v)
+    for (qsizetype i = 0, end = o->elements.length(); i < end; ++i) {
+        if (o->valueAt(i) != other.o->valueAt(i))
             return false;
     }
 
@@ -646,19 +658,12 @@ bool QJsonObject::operator!=(const QJsonObject &other) const
  */
 QJsonObject::iterator QJsonObject::erase(QJsonObject::iterator it)
 {
-    Q_ASSERT(d && d->ref.load() == 1);
-    if (it.o != this || it.i < 0 || it.i >= (int)o->length)
-        return iterator(this, o->length);
+    removeAt(it.item.index * 2);
 
-    int index = it.i;
-
-    o->removeItems(index, 1);
-    ++d->compactionCounter;
-    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(o->length) / 2u)
-        compact();
-
-    // iterator hasn't changed
-    return it;
+    // index hasn't changed; the container pointer shouldn't have changed
+    // because we shouldn't have detached (detaching happens on obtaining a
+    // non-const iterator). But just in case we did, reload the pointer.
+    return { this, qsizetype(it.item.index) };
 }
 
 /*!
@@ -670,26 +675,39 @@ QJsonObject::iterator QJsonObject::erase(QJsonObject::iterator it)
  */
 QJsonObject::iterator QJsonObject::find(const QString &key)
 {
-    bool keyExists = false;
-    int index = o ? o->indexOf(key, &keyExists) : 0;
-    if (!keyExists)
-        return end();
-    detach2();
-    return iterator(this, index);
+    return find(QStringView(key));
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonObject::iterator QJsonObject::find(QStringView key)
+{
+    return findImpl(key);
 }
 
 /*!
     \overload
     \since 5.7
 */
-QJsonObject::iterator QJsonObject::find(QLatin1String key)
+QJsonObject::iterator QJsonObject::find(QLatin1StringView key)
+{
+    return findImpl(key);
+}
+
+/*!
+    \internal
+*/
+template <typename T>
+QJsonObject::iterator QJsonObject::findImpl(T key)
 {
     bool keyExists = false;
-    int index = o ? o->indexOf(key, &keyExists) : 0;
+    auto index = o ? indexOf(o, key, &keyExists) : 0;
     if (!keyExists)
         return end();
-    detach2();
-    return iterator(this, index);
+    detach();
+    return {this, index / 2};
 }
 
 /*! \fn QJsonObject::const_iterator QJsonObject::find(const QString &key) const
@@ -697,7 +715,13 @@ QJsonObject::iterator QJsonObject::find(QLatin1String key)
     \overload
 */
 
-/*! \fn QJsonObject::const_iterator QJsonObject::find(QLatin1String key) const
+/*! \fn QJsonObject::const_iterator QJsonObject::find(QStringView key) const
+
+    \overload
+    \since 5.14
+*/
+
+/*! \fn QJsonObject::const_iterator QJsonObject::find(QLatin1StringView key) const
 
     \overload
     \since 5.7
@@ -712,34 +736,48 @@ QJsonObject::iterator QJsonObject::find(QLatin1String key)
  */
 QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
 {
-    bool keyExists = false;
-    int index = o ? o->indexOf(key, &keyExists) : 0;
-    if (!keyExists)
-        return end();
-    return const_iterator(this, index);
+    return constFind(QStringView(key));
+}
+
+/*!
+    \overload
+    \since 5.14
+*/
+QJsonObject::const_iterator QJsonObject::constFind(QStringView key) const
+{
+    return constFindImpl(key);
 }
 
 /*!
     \overload
     \since 5.7
 */
-QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
+QJsonObject::const_iterator QJsonObject::constFind(QLatin1StringView key) const
 {
-    bool keyExists = false;
-    int index = o ? o->indexOf(key, &keyExists) : 0;
-    if (!keyExists)
-        return end();
-    return const_iterator(this, index);
+    return constFindImpl(key);
 }
 
-/*! \fn int QJsonObject::count() const
+/*!
+    \internal
+*/
+template <typename T>
+QJsonObject::const_iterator QJsonObject::constFindImpl(T key) const
+{
+    bool keyExists = false;
+    auto index = o ? indexOf(o, key, &keyExists) : 0;
+    if (!keyExists)
+        return end();
+    return {this, index / 2};
+}
+
+/*! \fn qsizetype QJsonObject::count() const
 
     \overload
 
     Same as size().
 */
 
-/*! \fn int QJsonObject::length() const
+/*! \fn qsizetype QJsonObject::length() const
 
     \overload
 
@@ -863,7 +901,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa QJsonObject::begin(), QJsonObject::end()
 */
 
-/*! \fn QJsonObject::iterator::iterator(QJsonObject *obj, int index)
+/*! \fn QJsonObject::iterator::iterator(QJsonObject *obj, qsizetype index)
     \internal
 */
 
@@ -909,11 +947,33 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa key()
 */
 
-/*! \fn QJsonValueRef *QJsonObject::iterator::operator->() const
+/*! \fn QJsonValueRef *QJsonObject::iterator::operator->()
 
     Returns a pointer to a modifiable reference to the current item.
 */
 
+/*! \fn const QJsonValueConstRef *QJsonObject::iterator::operator->() const
+
+    Returns a pointer to a constant reference to the current item.
+*/
+
+/*! \fn const QJsonValueRef QJsonObject::iterator::operator[](qsizetype j) const
+
+    Returns a modifiable reference to the item at offset \a j from the
+    item pointed to by this iterator (the item at position \c{*this + j}).
+
+    This function is provided to make QJsonObject iterators behave like C++
+    pointers.
+
+    The return value is of type QJsonValueRef, a helper class for QJsonArray
+    and QJsonObject. When you get an object of type QJsonValueRef, you can
+    use it as if it were a reference to a QJsonValue. If you assign to it,
+    the assignment will apply to the element in the QJsonArray or QJsonObject
+    from which you got the reference.
+
+    \sa operator+()
+*/
+
 /*!
     \fn bool QJsonObject::iterator::operator==(const iterator &other) const
     \fn bool QJsonObject::iterator::operator==(const const_iterator &other) const
@@ -934,6 +994,38 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator==()
 */
 
+/*!
+    \fn bool QJsonObject::iterator::operator<(const iterator& other) const
+    \fn bool QJsonObject::iterator::operator<(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is less than
+    the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::iterator::operator<=(const iterator& other) const
+    \fn bool QJsonObject::iterator::operator<=(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is less than
+    or equal to the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::iterator::operator>(const iterator& other) const
+    \fn bool QJsonObject::iterator::operator>(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is greater
+    than the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::iterator::operator>=(const iterator& other) const
+    \fn bool QJsonObject::iterator::operator>=(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is greater
+    than or equal to the item pointed to by the \a other iterator.
+*/
+
 /*! \fn QJsonObject::iterator QJsonObject::iterator::operator++()
 
     The prefix ++ operator, \c{++i}, advances the iterator to the
@@ -974,7 +1066,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     current item.
 */
 
-/*! \fn QJsonObject::iterator QJsonObject::iterator::operator+(int j) const
+/*! \fn QJsonObject::iterator QJsonObject::iterator::operator+(qsizetype j) const
 
     Returns an iterator to the item at \a j positions forward from
     this iterator. If \a j is negative, the iterator goes backward.
@@ -983,7 +1075,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
 
 */
 
-/*! \fn QJsonObject::iterator QJsonObject::iterator::operator-(int j) const
+/*! \fn QJsonObject::iterator QJsonObject::iterator::operator-(qsizetype j) const
 
     Returns an iterator to the item at \a j positions backward from
     this iterator. If \a j is negative, the iterator goes forward.
@@ -991,7 +1083,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator+()
 */
 
-/*! \fn QJsonObject::iterator &QJsonObject::iterator::operator+=(int j)
+/*! \fn QJsonObject::iterator &QJsonObject::iterator::operator+=(qsizetype j)
 
     Advances the iterator by \a j items. If \a j is negative, the
     iterator goes backward.
@@ -999,7 +1091,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator-=(), operator+()
 */
 
-/*! \fn QJsonObject::iterator &QJsonObject::iterator::operator-=(int j)
+/*! \fn QJsonObject::iterator &QJsonObject::iterator::operator-=(qsizetype j)
 
     Makes the iterator go back by \a j items. If \a j is negative,
     the iterator goes forward.
@@ -1007,6 +1099,12 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator+=(), operator-()
 */
 
+/*! \fn qsizetype QJsonObject::iterator::operator-(iterator other) const
+
+    Returns the number of items between the item pointed to by \a
+    other and the item pointed to by this iterator.
+*/
+
 /*!
     \class QJsonObject::const_iterator
     \inmodule QtCore
@@ -1073,7 +1171,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa QJsonObject::constBegin(), QJsonObject::constEnd()
 */
 
-/*! \fn QJsonObject::const_iterator::const_iterator(const QJsonObject *obj, int index)
+/*! \fn QJsonObject::const_iterator::const_iterator(const QJsonObject *obj, qsizetype index)
     \internal
 */
 
@@ -1089,14 +1187,14 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa value()
 */
 
-/*! \fn QJsonValue QJsonObject::const_iterator::value() const
+/*! \fn QJsonValueConstRef QJsonObject::const_iterator::value() const
 
     Returns the current item's value.
 
     \sa key(), operator*()
 */
 
-/*! \fn QJsonValue QJsonObject::const_iterator::operator*() const
+/*! \fn const QJsonValueConstRef QJsonObject::const_iterator::operator*() const
 
     Returns the current item's value.
 
@@ -1105,11 +1203,23 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa key()
 */
 
-/*! \fn QJsonValue *QJsonObject::const_iterator::operator->() const
+/*! \fn const QJsonValueConstRef *QJsonObject::const_iterator::operator->() const
 
     Returns a pointer to the current item.
 */
 
+/*! \fn const QJsonValueConstRef QJsonObject::const_iterator::operator[](qsizetype j) const
+
+    Returns the item at offset \a j from the item pointed to by this iterator (the item at
+    position \c{*this + j}).
+
+    This function is provided to make QJsonObject iterators behave like C++
+    pointers.
+
+    \sa operator+()
+*/
+
+
 /*! \fn bool QJsonObject::const_iterator::operator==(const const_iterator &other) const
     \fn bool QJsonObject::const_iterator::operator==(const iterator &other) const
 
@@ -1128,6 +1238,34 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator==()
 */
 
+/*!
+    \fn bool QJsonObject::const_iterator::operator<(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is less than
+    the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::const_iterator::operator<=(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is less than
+    or equal to the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::const_iterator::operator>(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is greater
+    than the item pointed to by the \a other iterator.
+*/
+
+/*!
+    \fn bool QJsonObject::const_iterator::operator>=(const const_iterator& other) const
+
+    Returns \c true if the item pointed to by this iterator is greater
+    than or equal to the item pointed to by the \a other iterator.
+*/
+
 /*! \fn QJsonObject::const_iterator QJsonObject::const_iterator::operator++()
 
     The prefix ++ operator, \c{++i}, advances the iterator to the
@@ -1168,7 +1306,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     current item.
 */
 
-/*! \fn QJsonObject::const_iterator QJsonObject::const_iterator::operator+(int j) const
+/*! \fn QJsonObject::const_iterator QJsonObject::const_iterator::operator+(qsizetype j) const
 
     Returns an iterator to the item at \a j positions forward from
     this iterator. If \a j is negative, the iterator goes backward.
@@ -1178,7 +1316,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator-()
 */
 
-/*! \fn QJsonObject::const_iterator QJsonObject::const_iterator::operator-(int j) const
+/*! \fn QJsonObject::const_iterator QJsonObject::const_iterator::operator-(qsizetype j) const
 
     Returns an iterator to the item at \a j positions backward from
     this iterator. If \a j is negative, the iterator goes forward.
@@ -1188,7 +1326,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator+()
 */
 
-/*! \fn QJsonObject::const_iterator &QJsonObject::const_iterator::operator+=(int j)
+/*! \fn QJsonObject::const_iterator &QJsonObject::const_iterator::operator+=(qsizetype j)
 
     Advances the iterator by \a j items. If \a j is negative, the
     iterator goes backward.
@@ -1198,7 +1336,7 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator-=(), operator+()
 */
 
-/*! \fn QJsonObject::const_iterator &QJsonObject::const_iterator::operator-=(int j)
+/*! \fn QJsonObject::const_iterator &QJsonObject::const_iterator::operator-=(qsizetype j)
 
     Makes the iterator go back by \a j items. If \a j is negative,
     the iterator goes forward.
@@ -1208,91 +1346,71 @@ QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const
     \sa operator+=(), operator-()
 */
 
+/*! \fn qsizetype QJsonObject::const_iterator::operator-(const_iterator other) const
+
+    Returns the number of items between the item pointed to by \a
+    other and the item pointed to by this iterator.
+*/
+
 
 /*!
     \internal
  */
-void QJsonObject::detach(uint reserve)
+bool QJsonObject::detach(qsizetype reserve)
 {
-    Q_UNUSED(reserve)
-    Q_ASSERT(!reserve);
-    detach2(reserve);
-}
-
-bool QJsonObject::detach2(uint reserve)
-{
-    if (!d) {
-        if (reserve >= QJsonPrivate::Value::MaxSize) {
-            qWarning("QJson: Document too large to store in data structure");
-            return false;
-        }
-        d = new QJsonPrivate::Data(reserve, QJsonValue::Object);
-        o = static_cast<QJsonPrivate::Object *>(d->header->root());
-        d->ref.ref();
-        return true;
-    }
-    if (reserve == 0 && d->ref.load() == 1)
+    if (!o)
         return true;
+    o = QCborContainerPrivate::detach(o.data(), reserve ? reserve * 2 : o->elements.length());
+    return o;
+}
 
-    QJsonPrivate::Data *x = d->clone(o, reserve);
-    if (!x)
-        return false;
-    x->ref.ref();
-    if (!d->ref.deref())
-        delete d;
-    d = x;
-    o = static_cast<QJsonPrivate::Object *>(d->header->root());
-    return true;
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0) && !defined(QT_BOOTSTRAPPED)
+/*!
+    \internal
+ */
+QString QJsonObject::keyAt(qsizetype i) const
+{
+    Q_ASSERT(o && i >= 0 && i * 2 < o->elements.length());
+    return o->stringAt(i * 2);
 }
 
 /*!
     \internal
  */
-void QJsonObject::compact()
+QJsonValue QJsonObject::valueAt(qsizetype i) const
 {
-    if (!d || !d->compactionCounter)
-        return;
-
-    detach2();
-    d->compact();
-    o = static_cast<QJsonPrivate::Object *>(d->header->root());
-}
-
-/*!
-    \internal
- */
-QString QJsonObject::keyAt(int i) const
-{
-    Q_ASSERT(o && i >= 0 && i < (int)o->length);
-
-    QJsonPrivate::Entry *e = o->entryAt(i);
-    return e->key();
-}
-
-/*!
-    \internal
- */
-QJsonValue QJsonObject::valueAt(int i) const
-{
-    if (!o || i < 0 || i >= (int)o->length)
+    if (!o || i < 0 || 2 * i + 1 >= o->elements.length())
         return QJsonValue(QJsonValue::Undefined);
-
-    QJsonPrivate::Entry *e = o->entryAt(i);
-    return QJsonValue(d, o, e->value);
+    return QJsonPrivate::Value::fromTrustedCbor(o->valueAt(2 * i + 1));
 }
 
 /*!
     \internal
  */
-void QJsonObject::setValueAt(int i, const QJsonValue &val)
+void QJsonObject::setValueAt(qsizetype i, const QJsonValue &val)
 {
-    Q_ASSERT(o && i >= 0 && i < (int)o->length);
+    Q_ASSERT(o && i >= 0 && 2 * i + 1 < o->elements.length());
+    detach();
+    if (val.isUndefined()) {
+        o->removeAt(2 * i + 1);
+        o->removeAt(2 * i);
+    } else {
+        o->replaceAt(2 * i + 1, QCborValue::fromJsonValue(val));
+    }
+}
+#endif // Qt 7
 
-    QJsonPrivate::Entry *e = o->entryAt(i);
-    insert(e->key(), val);
+/*!
+    \internal
+ */
+void QJsonObject::removeAt(qsizetype index)
+{
+    detach();
+    o->removeAt(index + 1);
+    o->removeAt(index);
 }
 
-uint qHash(const QJsonObject &object, uint seed)
+size_t qHash(const QJsonObject &object, size_t seed)
 {
     QtPrivate::QHashCombine hash;
     for (auto it = object.begin(), end = object.end(); it != end; ++it) {
@@ -1312,7 +1430,7 @@ QDebug operator<<(QDebug dbg, const QJsonObject &o)
         return dbg;
     }
     QByteArray json;
-    QJsonPrivate::Writer::objectToJson(o.o, json, 0, true);
+    QJsonPrivate::Writer::objectToJson(o.o.data(), json, 0, true);
     dbg.nospace() << "QJsonObject("
                   << json.constData() // print as utf-8 string without extra quotation marks
                   << ")";
@@ -1320,4 +1438,21 @@ QDebug operator<<(QDebug dbg, const QJsonObject &o)
 }
 #endif
 
+#ifndef QT_NO_DATASTREAM
+QDataStream &operator<<(QDataStream &stream, const QJsonObject &object)
+{
+    QJsonDocument doc{object};
+    stream << doc.toJson(QJsonDocument::Compact);
+    return stream;
+}
+
+QDataStream &operator>>(QDataStream &stream, QJsonObject &object)
+{
+    QJsonDocument doc;
+    stream >> doc;
+    object = doc.object();
+    return stream;
+}
+#endif
+
 QT_END_NAMESPACE
