@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qkeysequence.h"
 #include "qkeysequence_p.h"
@@ -54,11 +18,14 @@
 #endif
 
 #include <algorithm>
+#include <q20algorithm.h>
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 #if defined(Q_OS_MACOS) || defined(Q_CLANG_QDOC)
-static bool qt_sequence_no_mnemonics = true;
+Q_CONSTINIT static bool qt_sequence_no_mnemonics = true;
 struct MacSpecialKey {
     int key;
     ushort macSymbol;
@@ -66,13 +33,12 @@ struct MacSpecialKey {
 
 // Unicode code points for the glyphs associated with these keys
 // Defined by Carbon headers but not anywhere in Cocoa
-static const int kShiftUnicode = 0x21E7;
-static const int kControlUnicode = 0x2303;
-static const int kOptionUnicode = 0x2325;
-static const int kCommandUnicode = 0x2318;
+static constexpr int kShiftUnicode = 0x21E7;
+static constexpr int kControlUnicode = 0x2303;
+static constexpr int kOptionUnicode = 0x2325;
+static constexpr int kCommandUnicode = 0x2318;
 
-static const int NumEntries = 21;
-static const MacSpecialKey entries[NumEntries] = {
+static constexpr MacSpecialKey entries[] = {
     { Qt::Key_Escape, 0x238B },
     { Qt::Key_Tab, 0x21E5 },
     { Qt::Key_Backtab, 0x21E4 },
@@ -80,6 +46,7 @@ static const MacSpecialKey entries[NumEntries] = {
     { Qt::Key_Return, 0x21B5 },
     { Qt::Key_Enter, 0x2324 },
     { Qt::Key_Delete, 0x2326 },
+    { Qt::Key_Clear, 0x2327 },
     { Qt::Key_Home, 0x2196 },
     { Qt::Key_End, 0x2198 },
     { Qt::Key_Left, 0x2190 },
@@ -93,24 +60,30 @@ static const MacSpecialKey entries[NumEntries] = {
     { Qt::Key_Meta, kControlUnicode },
     { Qt::Key_Alt, kOptionUnicode },
     { Qt::Key_CapsLock, 0x21EA },
+    { Qt::Key_Eject, 0x23CF },
 };
 
-static bool operator<(const MacSpecialKey &entry, int key)
+static constexpr bool operator<(const MacSpecialKey &lhs, const MacSpecialKey &rhs)
 {
-    return entry.key < key;
+    return lhs.key < rhs.key;
 }
 
-static bool operator<(int key, const MacSpecialKey &entry)
+static constexpr bool operator<(const MacSpecialKey &lhs, int rhs)
 {
-    return key < entry.key;
+    return lhs.key < rhs;
 }
 
-static const MacSpecialKey * const MacSpecialKeyEntriesEnd = entries + NumEntries;
+static constexpr bool operator<(int lhs, const MacSpecialKey &rhs)
+{
+    return lhs < rhs.key;
+}
+
+static_assert(q20::is_sorted(std::begin(entries), std::end(entries)));
 
 QChar qt_macSymbolForQtKey(int key)
 {
-    const MacSpecialKey *i = std::lower_bound(entries, MacSpecialKeyEntriesEnd, key);
-    if ((i == MacSpecialKeyEntriesEnd) || (key < *i))
+    const auto i = std::lower_bound(std::begin(entries), std::end(entries), key);
+    if (i == std::end(entries) || key < *i)
         return QChar();
     ushort macSymbol = i->macSymbol;
     if (qApp->testAttribute(Qt::AA_MacDontSwapCtrlAndMeta)
@@ -127,8 +100,7 @@ QChar qt_macSymbolForQtKey(int key)
 static int qtkeyForMacSymbol(const QChar ch)
 {
     const ushort unicode = ch.unicode();
-    for (int i = 0; i < NumEntries; ++i) {
-        const MacSpecialKey &entry = entries[i];
+    for (const MacSpecialKey &entry : entries) {
         if (entry.macSymbol == unicode) {
             int key = entry.key;
             if (qApp->testAttribute(Qt::AA_MacDontSwapCtrlAndMeta)
@@ -145,7 +117,7 @@ static int qtkeyForMacSymbol(const QChar ch)
 }
 
 #else
-static bool qt_sequence_no_mnemonics = false;
+Q_CONSTINIT static bool qt_sequence_no_mnemonics = false;
 #endif
 
 /*!
@@ -410,7 +382,7 @@ void Q_GUI_EXPORT qt_set_sequence_auto_mnemonic(bool b) { qt_sequence_no_mnemoni
     similar to the native text on Windows and X11.
 */
 
-static const struct {
+static constexpr struct {
     int key;
     const char name[25];
 } keyname[] = {
@@ -699,6 +671,10 @@ static const struct {
     { Qt::Key_TouchpadToggle,  QT_TRANSLATE_NOOP("QShortcut", "Touchpad Toggle") },
     { Qt::Key_TouchpadOn,  QT_TRANSLATE_NOOP("QShortcut", "Touchpad On") },
     { Qt::Key_TouchpadOff,  QT_TRANSLATE_NOOP("QShortcut", "Touchpad Off") },
+    { Qt::Key_Shift,  QT_TRANSLATE_NOOP("QShortcut", "Shift") },
+    { Qt::Key_Control,  QT_TRANSLATE_NOOP("QShortcut", "Control") },
+    { Qt::Key_Alt,  QT_TRANSLATE_NOOP("QShortcut", "Alt") },
+    { Qt::Key_Meta,  QT_TRANSLATE_NOOP("QShortcut", "Meta") },
 
 };
 static constexpr int numKeyNames = sizeof keyname / sizeof *keyname;
@@ -821,7 +797,7 @@ QKeySequence::QKeySequence(StandardKey key)
 */
 QKeySequence::QKeySequence()
 {
-    static QKeySequencePrivate shared_empty;
+    Q_CONSTINIT static QKeySequencePrivate shared_empty;
     d = &shared_empty;
     d->ref.ref();
 }
@@ -970,10 +946,10 @@ QKeySequence QKeySequence::mnemonic(const QString &text)
     bool found = false;
     int p = 0;
     while (p >= 0) {
-        p = text.indexOf(QLatin1Char('&'), p) + 1;
+        p = text.indexOf(u'&', p) + 1;
         if (p <= 0 || p >= (int)text.length())
             break;
-        if (text.at(p) != QLatin1Char('&')) {
+        if (text.at(p) != u'&') {
             QChar c = text.at(p);
             if (c.isPrint()) {
                 if (!found) {
@@ -1029,14 +1005,14 @@ int QKeySequence::assign(const QString &ks, QKeySequence::SequenceFormat format)
         // We MUST use something to separate each sequence, and space
         // does not cut it, since some of the key names have space
         // in them.. (Let's hope no one translate with a comma in it:)
-        p = keyseq.indexOf(QLatin1Char(','));
+        p = keyseq.indexOf(u',');
         if (-1 != p) {
-            if (p == keyseq.count() - 1) { // Last comma 'Ctrl+,'
+            if (p == keyseq.length() - 1) { // Last comma 'Ctrl+,'
                 p = -1;
             } else {
-                if (QLatin1Char(',') == keyseq.at(p+1)) // e.g. 'Ctrl+,, Shift+,,'
+                if (u',' == keyseq.at(p+1)) // e.g. 'Ctrl+,, Shift+,,'
                     p++;
-                if (QLatin1Char(' ') == keyseq.at(p+1)) { // Space after comma
+                if (u' ' == keyseq.at(p+1)) { // Space after comma
                     diff = 1;
                     p++;
                 } else {
@@ -1059,7 +1035,7 @@ struct QModifKeyName {
     int qt_key;
     QString name;
 };
-Q_DECLARE_TYPEINFO(QModifKeyName, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QModifKeyName, Q_RELOCATABLE_TYPE);
 
 Q_GLOBAL_STATIC(QList<QModifKeyName>, globalModifs)
 Q_GLOBAL_STATIC(QList<QModifKeyName>, globalPortableModifs)
@@ -1097,31 +1073,31 @@ int QKeySequencePrivate::decodeString(QString accel, QKeySequence::SequenceForma
                 *gmodifs << QModifKeyName(Qt::META, QChar(kControlUnicode));
             *gmodifs << QModifKeyName(Qt::SHIFT, QChar(kShiftUnicode));
 #endif
-            *gmodifs << QModifKeyName(Qt::CTRL, QLatin1String("ctrl+"))
-                     << QModifKeyName(Qt::SHIFT, QLatin1String("shift+"))
-                     << QModifKeyName(Qt::ALT, QLatin1String("alt+"))
-                     << QModifKeyName(Qt::META, QLatin1String("meta+"))
-                     << QModifKeyName(Qt::KeypadModifier, QLatin1String("num+"));
+            *gmodifs << QModifKeyName(Qt::CTRL, "ctrl+"_L1)
+                     << QModifKeyName(Qt::SHIFT, "shift+"_L1)
+                     << QModifKeyName(Qt::ALT, "alt+"_L1)
+                     << QModifKeyName(Qt::META, "meta+"_L1)
+                     << QModifKeyName(Qt::KeypadModifier, "num+"_L1);
         }
     } else {
         gmodifs = globalPortableModifs();
         if (gmodifs->isEmpty()) {
-            *gmodifs << QModifKeyName(Qt::CTRL, QLatin1String("ctrl+"))
-                     << QModifKeyName(Qt::SHIFT, QLatin1String("shift+"))
-                     << QModifKeyName(Qt::ALT, QLatin1String("alt+"))
-                     << QModifKeyName(Qt::META, QLatin1String("meta+"))
-                     << QModifKeyName(Qt::KeypadModifier, QLatin1String("num+"));
+            *gmodifs << QModifKeyName(Qt::CTRL, "ctrl+"_L1)
+                     << QModifKeyName(Qt::SHIFT, "shift+"_L1)
+                     << QModifKeyName(Qt::ALT, "alt+"_L1)
+                     << QModifKeyName(Qt::META, "meta+"_L1)
+                     << QModifKeyName(Qt::KeypadModifier, "num+"_L1);
         }
     }
 
 
     QList<QModifKeyName> modifs;
     if (nativeText) {
-        modifs << QModifKeyName(Qt::CTRL, QCoreApplication::translate("QShortcut", "Ctrl").toLower().append(QLatin1Char('+')))
-               << QModifKeyName(Qt::SHIFT, QCoreApplication::translate("QShortcut", "Shift").toLower().append(QLatin1Char('+')))
-               << QModifKeyName(Qt::ALT, QCoreApplication::translate("QShortcut", "Alt").toLower().append(QLatin1Char('+')))
-               << QModifKeyName(Qt::META, QCoreApplication::translate("QShortcut", "Meta").toLower().append(QLatin1Char('+')))
-               << QModifKeyName(Qt::KeypadModifier, QCoreApplication::translate("QShortcut", "Num").toLower().append(QLatin1Char('+')));
+        modifs << QModifKeyName(Qt::CTRL, QCoreApplication::translate("QShortcut", "Ctrl").toLower().append(u'+'))
+               << QModifKeyName(Qt::SHIFT, QCoreApplication::translate("QShortcut", "Shift").toLower().append(u'+'))
+               << QModifKeyName(Qt::ALT, QCoreApplication::translate("QShortcut", "Alt").toLower().append(u'+'))
+               << QModifKeyName(Qt::META, QCoreApplication::translate("QShortcut", "Meta").toLower().append(u'+'))
+               << QModifKeyName(Qt::KeypadModifier, QCoreApplication::translate("QShortcut", "Num").toLower().append(u'+'));
     }
     modifs += *gmodifs; // Test non-translated ones last
 
@@ -1141,7 +1117,7 @@ int QKeySequencePrivate::decodeString(QString accel, QKeySequence::SequenceForma
 
     int i = 0;
     int lastI = 0;
-    while ((i = sl.indexOf(QLatin1Char('+'), i + 1)) != -1) {
+    while ((i = sl.indexOf(u'+', i + 1)) != -1) {
         const QStringView sub = QStringView{sl}.mid(lastI, i - lastI + 1);
         // If we get here the shortcuts contains at least one '+'. We break up
         // along the following strategy:
@@ -1154,7 +1130,7 @@ int QKeySequencePrivate::decodeString(QString accel, QKeySequence::SequenceForma
         // Only '+' can have length 1.
         if (sub.length() == 1) {
             // Make sure we only encounter a single '+' at the end of the accel
-            if (accel.lastIndexOf(QLatin1Char('+')) != accel.length()-1)
+            if (accel.lastIndexOf(u'+') != accel.length()-1)
                 return Qt::Key_unknown;
         } else {
             // Identify the modifier
@@ -1174,7 +1150,7 @@ int QKeySequencePrivate::decodeString(QString accel, QKeySequence::SequenceForma
         lastI = i + 1;
     }
 
-    int p = accel.lastIndexOf(QLatin1Char('+'), accel.length() - 2); // -2 so that Ctrl++ works
+    int p = accel.lastIndexOf(u'+', accel.length() - 2); // -2 so that Ctrl++ works
     QStringView accelRef(accel);
     if (p > 0)
         accelRef = accelRef.mid(p + 1);
@@ -1190,10 +1166,10 @@ int QKeySequencePrivate::decodeString(QString accel, QKeySequence::SequenceForma
         {
             ret |= accelRef.at(0).toUpper().unicode();
         }
-    } else if (accelRef.at(0) == QLatin1Char('f') && (fnum = accelRef.mid(1).toInt()) >= 1 && fnum <= 35) {
+    } else if (accelRef.at(0) == u'f' && (fnum = accelRef.mid(1).toInt()) >= 1 && fnum <= 35) {
         ret |= Qt::Key_F1 + fnum - 1;
     } else {
-        // For NativeText, check the traslation table first,
+        // For NativeText, check the translation table first,
         // if we don't find anything then try it out with just the untranlated stuff.
         // PortableText will only try the untranlated table.
         bool found = false;
@@ -1237,7 +1213,7 @@ static inline void addKey(QString &str, const QString &theKey, QKeySequence::Seq
             //: Key separator in shortcut string
             str += QCoreApplication::translate("QShortcut", "+");
         } else {
-            str += QLatin1Char('+');
+            str += u'+';
         }
     }
 
@@ -1261,10 +1237,10 @@ QString QKeySequencePrivate::encodeString(int key, QKeySequence::SequenceFormat
         // for us, which means that we have to adjust our order here.
         // The upshot is a lot more infrastructure to keep the number of
         // if tests down and the code relatively clean.
-        static const int ModifierOrder[] = { Qt::META, Qt::ALT, Qt::SHIFT, Qt::CTRL, 0 };
-        static const int QtKeyOrder[] = { Qt::Key_Meta, Qt::Key_Alt, Qt::Key_Shift, Qt::Key_Control, 0 };
-        static const int DontSwapModifierOrder[] = { Qt::CTRL, Qt::ALT, Qt::SHIFT, Qt::META, 0 };
-        static const int DontSwapQtKeyOrder[] = { Qt::Key_Control, Qt::Key_Alt, Qt::Key_Shift, Qt::Key_Meta, 0 };
+        static constexpr int ModifierOrder[] = { Qt::META, Qt::ALT, Qt::SHIFT, Qt::CTRL, 0 };
+        static constexpr int QtKeyOrder[] = { Qt::Key_Meta, Qt::Key_Alt, Qt::Key_Shift, Qt::Key_Control, 0 };
+        static constexpr int DontSwapModifierOrder[] = { Qt::CTRL, Qt::ALT, Qt::SHIFT, Qt::META, 0 };
+        static constexpr int DontSwapQtKeyOrder[] = { Qt::Key_Control, Qt::Key_Alt, Qt::Key_Shift, Qt::Key_Meta, 0 };
         const int *modifierOrder;
         const int *qtkeyOrder;
         if (qApp->testAttribute(Qt::AA_MacDontSwapCtrlAndMeta)) {
@@ -1546,7 +1522,7 @@ QString QKeySequence::toString(SequenceFormat format) const
     int end = count();
     for (int i = 0; i < end; ++i) {
         finalString += d->encodeString(d->key[i], format);
-        finalString += QLatin1String(", ");
+        finalString += ", "_L1;
     }
     finalString.truncate(finalString.length() - 2);
     return finalString;
@@ -1576,7 +1552,7 @@ QList<QKeySequence> QKeySequence::listFromString(const QString &str, SequenceFor
 {
     QList<QKeySequence> result;
 
-    const QStringList strings = str.split(QLatin1String("; "));
+    const QStringList strings = str.split("; "_L1);
     result.reserve(strings.count());
     for (const QString &string : strings) {
         result << fromString(string, format);
@@ -1599,7 +1575,7 @@ QString QKeySequence::listToString(const QList<QKeySequence> &list, SequenceForm
 
     for (const QKeySequence &sequence : list) {
         result += sequence.toString(format);
-        result += QLatin1String("; ");
+        result += "; "_L1;
     }
     result.truncate(result.length() - 2);
 
@@ -1680,3 +1656,5 @@ QDebug operator<<(QDebug dbg, const QKeySequence &p)
 */
 
 QT_END_NAMESPACE
+
+#include "moc_qkeysequence.cpp"
