@@ -1,32 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2013 David Faure <faure@kde.org>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 David Faure <faure@kde.org>
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include <QtTest/QtTest>
+#include <QTest>
+#if QT_CONFIG(process)
+#include <QProcess>
+#endif
 #include <QtCore/QCommandLineParser>
 
 Q_DECLARE_METATYPE(char**)
@@ -44,6 +23,7 @@ private slots:
 
     // In-process tests
     void testInvalidOptions();
+    void testDuplicateOption();
     void testPositionalArguments();
     void testBooleanOption_data();
     void testBooleanOption();
@@ -62,6 +42,7 @@ private slots:
     void testDefaultValue();
     void testProcessNotCalled();
     void testEmptyArgsList();
+    void testNoApplication();
     void testMissingOptionValue();
     void testStdinArgument_data();
     void testStdinArgument();
@@ -75,6 +56,9 @@ private slots:
     void testHelpOption();
     void testQuoteEscaping();
     void testUnknownOption();
+    void testHelpAll_data();
+    void testHelpAll();
+    void testVeryLongOptionNames();
 };
 
 static char *empty_argv[] = { 0 };
@@ -104,6 +88,15 @@ void tst_QCommandLineParser::testInvalidOptions()
     QVERIFY(!parser.addOption(QCommandLineOption(QStringLiteral("-v"), QStringLiteral("Displays version information."))));
 }
 
+void tst_QCommandLineParser::testDuplicateOption()
+{
+    QCoreApplication app(empty_argc, empty_argv);
+    QCommandLineParser parser;
+    QVERIFY(parser.addOption(QCommandLineOption(QStringLiteral("h"), QStringLiteral("Hostname."), QStringLiteral("hostname"))));
+    QTest::ignoreMessage(QtWarningMsg, "QCommandLineParser: already having an option named \"h\"");
+    parser.addHelpOption();
+}
+
 void tst_QCommandLineParser::testPositionalArguments()
 {
     QCoreApplication app(empty_argc, empty_argv);
@@ -379,6 +372,34 @@ void tst_QCommandLineParser::testEmptyArgsList()
     QVERIFY(!parser.parse(QStringList())); // invalid call, argv[0] is missing
 }
 
+void tst_QCommandLineParser::testNoApplication()
+{
+    QCommandLineOption option(QStringLiteral("param"), QStringLiteral("Pass parameter to the backend."));
+    option.setValueName("key=value");
+    QCommandLineParser parser;
+    QVERIFY(parser.addOption(option));
+    {
+        QVERIFY(parser.parse(QStringList() << "tst" << "--param" << "key1=value1"));
+        QVERIFY(parser.isSet("param"));
+        QCOMPARE(parser.values("param"), QStringList() << "key1=value1");
+        QCOMPARE(parser.value("param"), QString("key1=value1"));
+    }
+    {
+        QVERIFY(parser.parse(QStringList() << "tst" << "--param" << "key1=value1" << "--param" << "key2=value2"));
+        QVERIFY(parser.isSet("param"));
+        QCOMPARE(parser.values("param"), QStringList() << "key1=value1" << "key2=value2");
+        QCOMPARE(parser.value("param"), QString("key2=value2"));
+    }
+
+    const QString expected =
+        "Usage: <executable_name> [options]\n"
+        "\n"
+        "Options:\n"
+        "  --param <key=value>  Pass parameter to the backend.\n";
+
+    QCOMPARE(parser.helpText(), expected);
+}
+
 void tst_QCommandLineParser::testMissingOptionValue()
 {
     QCoreApplication app(empty_argc, empty_argv);
@@ -489,7 +510,7 @@ void tst_QCommandLineParser::testSingleDashWordOptionModes()
 
 void tst_QCommandLineParser::testCpp11StyleInitialization()
 {
-#if defined(Q_COMPILER_INITIALIZER_LISTS) && defined(Q_COMPILER_UNIFORM_INIT)
+#if defined(Q_COMPILER_UNIFORM_INIT)
     QCoreApplication app(empty_argc, empty_argv);
 
     QCommandLineParser parser;
@@ -512,10 +533,9 @@ void tst_QCommandLineParser::testVersionOption()
 {
 #if !QT_CONFIG(process)
     QSKIP("This test requires QProcess support");
-#else
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#elif defined(Q_OS_ANDROID)
     QSKIP("Deploying executable applications to file system on Android not supported.");
-#endif
+#else
 
     QCoreApplication app(empty_argc, empty_argv);
     QProcess process;
@@ -532,7 +552,8 @@ void tst_QCommandLineParser::testVersionOption()
 
 static const char expectedOptionsHelp[] =
         "Options:\n"
-        "  -h, --help                  Displays this help.\n"
+        "  -h, --help                  Displays help on commandline options.\n"
+        "  --help-all                  Displays help including Qt specific options.\n"
         "  -v, --version               Displays version information.\n"
         "  --load <url>                Load file from URL.\n"
         "  -o, --output <file>         Set output file.\n"
@@ -566,8 +587,8 @@ void tst_QCommandLineParser::testHelpOption_data()
         "  parsingMode                 The parsing mode to test.\n"
         "  command                     The command to execute.\n");
 #ifdef Q_OS_WIN
-    expectedOutput.replace("  -h, --help                  Displays this help.\n",
-                           "  -?, -h, --help              Displays this help.\n");
+    expectedOutput.replace("  -h, --help                  Displays help on commandline options.\n",
+                           "  -?, -h, --help              Displays help on commandline options.\n");
     expectedOutput.replace("testhelper/", "testhelper\\");
 #endif
 
@@ -579,10 +600,9 @@ void tst_QCommandLineParser::testHelpOption()
 {
 #if !QT_CONFIG(process)
     QSKIP("This test requires QProcess support");
-#else
-#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#elif defined(Q_OS_ANDROID)
     QSKIP("Deploying executable applications to file system on Android not supported.");
-#endif
+#else
 
     QFETCH(QCommandLineParser::SingleDashWordOptionMode, parsingMode);
     QFETCH(QString, expectedHelpOutput);
@@ -615,8 +635,8 @@ void tst_QCommandLineParser::testHelpOption()
         "Arguments:\n"
         "  resize                      Resize the object to a new size.\n";
 #ifdef Q_OS_WIN
-    expectedResizeHelp.replace("  -h, --help                  Displays this help.\n",
-                               "  -?, -h, --help              Displays this help.\n");
+    expectedResizeHelp.replace("  -h, --help                  Displays help on commandline options.\n",
+                               "  -?, -h, --help              Displays help on commandline options.\n");
     expectedResizeHelp.replace("testhelper/", "testhelper\\");
 #endif
     QCOMPARE(output, QString(expectedResizeHelp));
@@ -627,7 +647,7 @@ void tst_QCommandLineParser::testQuoteEscaping()
 {
 #if !QT_CONFIG(process)
     QSKIP("This test requires QProcess support");
-#elif defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#elif defined(Q_OS_ANDROID)
     QSKIP("Deploying executable applications to file system on Android not supported.");
 #else
     QCoreApplication app(empty_argc, empty_argv);
@@ -653,7 +673,7 @@ void tst_QCommandLineParser::testUnknownOption()
 {
 #if !QT_CONFIG(process)
     QSKIP("This test requires QProcess support");
-#elif defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
+#elif defined(Q_OS_ANDROID)
     QSKIP("Deploying executable applications to file system on Android not supported.");
 #else
     QCoreApplication app(empty_argc, empty_argv);
@@ -670,6 +690,91 @@ void tst_QCommandLineParser::testUnknownOption()
 #endif // QT_CONFIG(process)
 }
 
+void tst_QCommandLineParser::testHelpAll_data()
+{
+    QTest::addColumn<QCommandLineParser::SingleDashWordOptionMode>("parsingMode");
+    QTest::addColumn<QString>("expectedHelpOutput");
+
+    QString expectedOutput = QString::fromLatin1(
+        "Usage: testhelper/qcommandlineparser_test_helper [options] parsingMode command\n"
+        "Test helper\n"
+        "\n")
+        + QString::fromLatin1(expectedOptionsHelp) +
+        QString::fromLatin1(
+        "  --qmljsdebugger <value>     Activates the QML/JS debugger with a specified\n"
+        "                              port. The value must be of format\n"
+        "                              port:1234[,block]. \"block\" makes the application\n"
+        "                              wait for a connection.\n"
+        "\n"
+        "Arguments:\n"
+        "  parsingMode                 The parsing mode to test.\n"
+        "  command                     The command to execute.\n");
+#ifdef Q_OS_WIN
+    expectedOutput.replace("  -h, --help                  Displays help on commandline options.\n",
+                           "  -?, -h, --help              Displays help on commandline options.\n");
+    expectedOutput.replace("testhelper/", "testhelper\\");
+#endif
+
+    QTest::newRow("collapsed") << QCommandLineParser::ParseAsCompactedShortOptions << expectedOutput;
+    QTest::newRow("long")      << QCommandLineParser::ParseAsLongOptions << expectedOutput;
+}
+
+void tst_QCommandLineParser::testHelpAll()
+{
+#if !QT_CONFIG(process)
+    QSKIP("This test requires QProcess support");
+#else
+#ifdef Q_OS_ANDROID
+    QSKIP("Deploying executable applications to file system on Android not supported.");
+#endif
+
+    QFETCH(QCommandLineParser::SingleDashWordOptionMode, parsingMode);
+    QFETCH(QString, expectedHelpOutput);
+    QCoreApplication app(empty_argc, empty_argv);
+    QProcess process;
+    process.start("testhelper/qcommandlineparser_test_helper", QStringList() << QString::number(parsingMode) << "--help-all");
+    QVERIFY(process.waitForFinished(5000));
+    QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+    QString output = process.readAll();
+#ifdef Q_OS_WIN
+    output.replace(QStringLiteral("\r\n"), QStringLiteral("\n"));
+#endif
+    QCOMPARE(output.split('\n'), expectedHelpOutput.split('\n')); // easier to debug than the next line, on failure
+    QCOMPARE(output, expectedHelpOutput);
+#endif // QT_CONFIG(process)
+}
+
+void tst_QCommandLineParser::testVeryLongOptionNames()
+{
+#if !QT_CONFIG(process)
+    QSKIP("This test requires QProcess support");
+#elif defined(Q_OS_ANDROID)
+    QSKIP("Deploying executable applications to file system on Android not supported.");
+#else
+
+    QCoreApplication app(empty_argc, empty_argv);
+    QProcess process;
+    process.start("testhelper/qcommandlineparser_test_helper", QStringList() << "0" << "long" << "--help");
+    QVERIFY(process.waitForFinished(5000));
+    QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+    QString output = process.readAll();
+#ifdef Q_OS_WIN
+    output.replace(QStringLiteral("\r\n"), QStringLiteral("\n"));
+#endif
+    const QStringList lines = output.split('\n');
+    const int last = lines.count() - 1;
+    // Let's not compare everything, just the final parts.
+    QCOMPARE(lines.at(last - 7), "                                                     cdefghijklmnopqrstuvwxyz");
+    QCOMPARE(lines.at(last - 6), "  --looooooooooooong-option, --looooong-opt-alias <l Short description");
+    QCOMPARE(lines.at(last - 5), "  ooooooooooooong-value-name>");
+    QCOMPARE(lines.at(last - 4), "");
+    QCOMPARE(lines.at(last - 3), "Arguments:");
+    QCOMPARE(lines.at(last - 2), "  parsingMode                                        The parsing mode to test.");
+    QCOMPARE(lines.at(last - 1), "  command                                            The command to execute.");
+
+#endif // QT_CONFIG(process)
+}
+
 QTEST_APPLESS_MAIN(tst_QCommandLineParser)
 #include "tst_qcommandlineparser.moc"
 
