@@ -1,33 +1,9 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QSignalSpy>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QWindow>
 #include <QtGui/QScreen>
@@ -76,6 +52,7 @@ private slots:
     void quitOnLastWindowClosed();
     void quitOnLastWindowClosedMulti();
     void dontQuitOnLastWindowClosed();
+    void quitOnLastWindowClosedWithEventLoopLocker();
     void genericPluginsAndWindowSystemEvents();
     void layoutDirection();
     void globalShareContext();
@@ -543,26 +520,35 @@ void tst_QGuiApplication::palette()
     // The default application palette is not resolved
     QVERIFY(!QGuiApplication::palette().resolveMask());
 
+    // TODO: add event processing instead of the signal
+#if QT_DEPRECATED_SINCE(6, 0)
     QSignalSpy signalSpy(&app, SIGNAL(paletteChanged(QPalette)));
+#endif
 
     QPalette oldPalette = QGuiApplication::palette();
     QPalette newPalette = QPalette(Qt::red);
 
     QGuiApplication::setPalette(newPalette);
     QVERIFY(palettesMatch(QGuiApplication::palette(), newPalette));
+#if QT_DEPRECATED_SINCE(6, 0)
     QCOMPARE(signalSpy.count(), 1);
+#endif
     QVERIFY(palettesMatch(signalSpy.at(0).at(0).value<QPalette>(), newPalette));
     QCOMPARE(QGuiApplication::palette(), QPalette());
 
     QGuiApplication::setPalette(oldPalette);
     QVERIFY(palettesMatch(QGuiApplication::palette(), oldPalette));
+#if QT_DEPRECATED_SINCE(6, 0)
     QCOMPARE(signalSpy.count(), 2);
+#endif
     QVERIFY(palettesMatch(signalSpy.at(1).at(0).value<QPalette>(), oldPalette));
     QCOMPARE(QGuiApplication::palette(), QPalette());
 
     QGuiApplication::setPalette(oldPalette);
     QVERIFY(palettesMatch(QGuiApplication::palette(), oldPalette));
+#if QT_DEPRECATED_SINCE(6, 0)
     QCOMPARE(signalSpy.count(), 2);
+#endif
     QCOMPARE(QGuiApplication::palette(), QPalette());
 }
 
@@ -904,11 +890,7 @@ void tst_QGuiApplication::quitOnLastWindowClosedMulti()
     QGuiApplication app(argc, nullptr);
     const QRect screenGeometry = QGuiApplication::primaryScreen()->availableVirtualGeometry();
 
-    QTimer timer;
-    timer.setInterval(100);
-
     QSignalSpy spyAboutToQuit(&app, &QCoreApplication::aboutToQuit);
-    QSignalSpy spyTimeout(&timer, &QTimer::timeout);
 
     QWindow mainWindow;
     mainWindow.setTitle(QStringLiteral("quitOnLastWindowClosedMultiMainWindow"));
@@ -927,15 +909,20 @@ void tst_QGuiApplication::quitOnLastWindowClosedMulti()
     dialog.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dialog));
 
-    timer.start();
-    QTimer::singleShot(1000, &mainWindow, &QWindow::close); // This should not quit the application
-    QTimer::singleShot(2000, &app, &QCoreApplication::quit);
+    bool prematureQuit = true;
+    QTimer::singleShot(100, &mainWindow, [&]{
+        prematureQuit = true; // this should be reset by the other timer
+        mainWindow.close();
+    });
+    QTimer::singleShot(500, &mainWindow, [&]{
+        prematureQuit = false; // if we don't get here, then the app quit prematurely
+        dialog.close();
+    });
 
     app.exec();
 
-    QCOMPARE(spyAboutToQuit.count(), 1);
-    // Should be around 20 if closing did not cause the quit
-    QVERIFY2(spyTimeout.count() > 15, QByteArray::number(spyTimeout.count()).constData());
+    QVERIFY(!prematureQuit);
+    QCOMPARE(spyAboutToQuit.count(), 1); // fired only once
 }
 
 void tst_QGuiApplication::dontQuitOnLastWindowClosed()
@@ -967,6 +954,121 @@ void tst_QGuiApplication::dontQuitOnLastWindowClosed()
     QCOMPARE(spyLastWindowClosed.count(), 1); // lastWindowClosed emitted
 }
 
+class QuitSpy : public QObject
+{
+    Q_OBJECT
+public:
+    QuitSpy()
+    {
+        qGuiApp->installEventFilter(this);
+    }
+    bool eventFilter(QObject *o, QEvent *e) override
+    {
+        Q_UNUSED(o);
+        if (e->type() == QEvent::Quit)
+            ++quits;
+
+        return false;
+    }
+
+    int quits = 0;
+};
+
+void tst_QGuiApplication::quitOnLastWindowClosedWithEventLoopLocker()
+{
+    int argc = 0;
+    QGuiApplication app(argc, nullptr);
+
+    QVERIFY(app.quitOnLastWindowClosed());
+    QVERIFY(app.isQuitLockEnabled());
+
+    auto defaultRestorer = qScopeGuard([&]{
+        app.setQuitLockEnabled(true);
+        app.setQuitOnLastWindowClosed(true);
+    });
+
+    {
+        // Disabling QEventLoopLocker support should not affect
+        // quitting when last window is closed.
+        app.setQuitLockEnabled(false);
+
+        QuitSpy quitSpy;
+        QWindow window;
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, &window, &QWindow::close);
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 1);
+    }
+
+    {
+        // Disabling quitOnLastWindowClosed support should not affect
+        // quitting when last QEventLoopLocker goes out of scope.
+        app.setQuitLockEnabled(true);
+        app.setQuitOnLastWindowClosed(false);
+
+        QuitSpy quitSpy;
+        QScopedPointer<QEventLoopLocker> locker(new QEventLoopLocker);
+        QWindow window;
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, [&]{ locker.reset(nullptr); });
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 1);
+    }
+
+    {
+        // With both properties enabled we need to get rid of both
+        // the window and locker to trigger a quit.
+        app.setQuitLockEnabled(true);
+        app.setQuitOnLastWindowClosed(true);
+
+        QuitSpy quitSpy;
+        QScopedPointer<QEventLoopLocker> locker(new QEventLoopLocker);
+        QWindow window;
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, &window, &QWindow::close);
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 0);
+
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, [&]{ locker.reset(nullptr); });
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 0);
+
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, [&]{ locker.reset(nullptr); });
+        QTimer::singleShot(0, &window, &QWindow::close);
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 1);
+    }
+
+    {
+        // With neither properties enabled we don't get automatic quit.
+        app.setQuitLockEnabled(false);
+        app.setQuitOnLastWindowClosed(false);
+
+        QuitSpy quitSpy;
+        QScopedPointer<QEventLoopLocker> locker(new QEventLoopLocker);
+        QWindow window;
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+        QTimer::singleShot(0, [&]{ locker.reset(nullptr); });
+        QTimer::singleShot(0, &window, &QWindow::close);
+        QTimer::singleShot(200, &app, []{ QCoreApplication::exit(0); });
+        app.exec();
+        QCOMPARE(quitSpy.quits, 0);
+    }
+}
+
 static Qt::ScreenOrientation testOrientationToSend = Qt::PrimaryOrientation;
 
 class TestPlugin : public QObject
@@ -1019,13 +1121,7 @@ void tst_QGuiApplication::genericPluginsAndWindowSystemEvents()
     QCoreApplication::postEvent(&testReceiver, new QEvent(QEvent::User));
     QCOMPARE(testReceiver.customEvents, 0);
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-    QStaticPlugin testPluginInfo(qt_plugin_instance, qt_plugin_query_metadata);
-#else
-    QStaticPlugin testPluginInfo;
-    testPluginInfo.instance = qt_plugin_instance;
-    testPluginInfo.rawMetaData = qt_plugin_query_metadata;
-#endif
+    QStaticPlugin testPluginInfo(qt_plugin_instance, qt_plugin_query_metadata_v2);
     qRegisterStaticPluginFunction(testPluginInfo);
     int argc = 3;
     char *argv[] = { const_cast<char*>(QTest::currentAppName()), const_cast<char*>("-plugin"), const_cast<char*>("testplugin") };
@@ -1044,8 +1140,8 @@ void tst_QGuiApplication::layoutDirection()
 {
     qRegisterMetaType<Qt::LayoutDirection>();
 
-    Qt::LayoutDirection oldDirection = QGuiApplication::layoutDirection();
-    Qt::LayoutDirection newDirection = oldDirection == Qt::LeftToRight ? Qt::RightToLeft : Qt::LeftToRight;
+    const Qt::LayoutDirection oldDirection = QGuiApplication::layoutDirection();
+    const Qt::LayoutDirection newDirection = oldDirection == Qt::LeftToRight ? Qt::RightToLeft : Qt::LeftToRight;
 
     QGuiApplication::setLayoutDirection(newDirection);
     QCOMPARE(QGuiApplication::layoutDirection(), newDirection);
@@ -1063,8 +1159,64 @@ void tst_QGuiApplication::layoutDirection()
     QGuiApplication::setLayoutDirection(oldDirection);
     QCOMPARE(QGuiApplication::layoutDirection(), oldDirection);
     QCOMPARE(signalSpy.count(), 1);
+
+    // with QGuiApplication instantiated, install a translator that gives us control
+    class LayoutDirectionTranslator : public QTranslator
+    {
+    public:
+        LayoutDirectionTranslator(Qt::LayoutDirection direction)
+        : direction(direction)
+        {}
+
+        bool isEmpty() const override { return false; }
+        QString translate(const char *context, const char *sourceText, const char *disambiguation, int n) const override
+        {
+            if (QByteArrayView(sourceText) == "QT_LAYOUT_DIRECTION")
+                return direction == Qt::LeftToRight ? QLatin1String("LTR") : QLatin1String("RTL");
+            return QTranslator::translate(context, sourceText, disambiguation, n);
+        }
+
+        const Qt::LayoutDirection direction;
+    };
+
+    int layoutDirectionChangedCount = 0;
+    // reset to auto-detection, should be back to oldDirection now
+    QGuiApplication::setLayoutDirection(Qt::LayoutDirectionAuto);
+    QCOMPARE(QGuiApplication::layoutDirection(), oldDirection);
+    signalSpy.clear();
+    {
+        // this translator doesn't change the direction
+        LayoutDirectionTranslator translator(oldDirection);
+        QGuiApplication::installTranslator(&translator);
+        QCOMPARE(QGuiApplication::layoutDirection(), translator.direction);
+        QCOMPARE(signalSpy.count(), layoutDirectionChangedCount);
+    }
+    QCOMPARE(signalSpy.count(), layoutDirectionChangedCount); // ltrTranslator removed, no change
+
+    // install a new translator that changes the direction
+    {
+        LayoutDirectionTranslator translator(newDirection);
+        QGuiApplication::installTranslator(&translator);
+        QCOMPARE(QGuiApplication::layoutDirection(), translator.direction);
+        QCOMPARE(signalSpy.count(), ++layoutDirectionChangedCount);
+    }
+    // rtlTranslator removed
+    QCOMPARE(signalSpy.count(), ++layoutDirectionChangedCount);
+
+    // override translation
+    QGuiApplication::setLayoutDirection(newDirection);
+    QCOMPARE(signalSpy.count(), ++layoutDirectionChangedCount);
+    {
+        // this translator will be ignored
+        LayoutDirectionTranslator translator(oldDirection);
+        QGuiApplication::installTranslator(&translator);
+        QCOMPARE(QGuiApplication::layoutDirection(), newDirection);
+        QCOMPARE(signalSpy.count(), layoutDirectionChangedCount);
+    }
+    QCOMPARE(signalSpy.count(), layoutDirectionChangedCount);
 }
 
+
 void tst_QGuiApplication::globalShareContext()
 {
 #ifndef QT_NO_OPENGL
@@ -1119,16 +1271,23 @@ void tst_QGuiApplication::staticFunctions()
     QGuiApplication::setWindowIcon(QIcon());
     QGuiApplication::windowIcon();
     QGuiApplication::platformName();
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::modalWindow();
     QGuiApplication::focusWindow();
     QGuiApplication::focusObject();
     QGuiApplication::primaryScreen();
     QGuiApplication::screens();
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::overrideCursor();
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::setOverrideCursor(QCursor());
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::changeOverrideCursor(QCursor());
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::restoreOverrideCursor();
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::keyboardModifiers();
+    QTest::ignoreMessage(QtWarningMsg, "Must construct a QGuiApplication first.");
     QGuiApplication::queryKeyboardModifiers();
     QGuiApplication::mouseButtons();
     QGuiApplication::setLayoutDirection(Qt::LeftToRight);
@@ -1138,11 +1297,13 @@ void tst_QGuiApplication::staticFunctions()
     QGuiApplication::desktopSettingsAware();
     QGuiApplication::inputMethod();
     QGuiApplication::platformNativeInterface();
+    QTest::ignoreMessage(QtWarningMsg, "QGuiApplication::platformFunction(): Must construct a QGuiApplication before accessing a platform function");
     QGuiApplication::platformFunction(QByteArrayLiteral("bla"));
     QGuiApplication::setQuitOnLastWindowClosed(true);
     QGuiApplication::quitOnLastWindowClosed();
     QGuiApplication::applicationState();
 
+    QTest::ignoreMessage(QtWarningMsg, "QPixmap: QGuiApplication must be created before calling defaultDepth().");
     QPixmap::defaultDepth();
 }
 
