@@ -1,34 +1,16 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2017 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2017 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QSignalSpy>
+#include <QtEndian>
+#if QT_CONFIG(process)
+#include <QProcess>
+#endif
+#include <QScopeGuard>
+#include <QVersionNumber>
 
 #include <qcoreapplication.h>
 #include <qfileinfo.h>
@@ -40,13 +22,15 @@
 #include <qhostinfo.h>
 #include <qtcpsocket.h>
 #include <qmap.h>
+#include <qelapsedtimer.h>
 #include <qnetworkdatagram.h>
 #include <QNetworkProxy>
 #include <QNetworkInterface>
 
 #include <qstringlist.h>
+#include <QSet>
 #include "../../../network-settings.h"
-#include "emulationdetector.h"
+#include <QtTest/private/qemulationdetector_p.h>
 
 #if defined(Q_OS_LINUX)
 #define SHOULD_CHECK_SYSCALL_SUPPORT
@@ -136,7 +120,7 @@ private:
     bool m_workaroundLinuxKernelBug;
     QList<QHostAddress> allAddresses;
     QHostAddress multicastGroup4, multicastGroup6;
-    QVector<QHostAddress> linklocalMulticastGroups;
+    QList<QHostAddress> linklocalMulticastGroups;
     QUdpSocket *m_asyncSender;
     QUdpSocket *m_asyncReceiver;
 };
@@ -187,7 +171,7 @@ QNetworkInterface tst_QUdpSocket::interfaceForGroup(const QHostAddress &multicas
     if (!scope.isEmpty())
         return QNetworkInterface::interfaceFromName(scope);
 
-    static QNetworkInterface ipv6if = [=]() {
+    static QNetworkInterface ipv6if = [&]() {
         // find any link local address in the allAddress list
         for (const QHostAddress &addr: qAsConst(allAddresses)) {
             if (addr.isLoopback())
@@ -235,6 +219,7 @@ void tst_QUdpSocket::initTestCase_data()
     // hack: we only enable the Socks5 over UDP tests on the old
     // test server, because they fail on the new one. See QTBUG-35490
     bool newTestServer = true;
+#ifndef QT_TEST_SERVER
     QTcpSocket socket;
     socket.connectToHost(QtNetworkSettings::serverName(), 22);
     if (socket.waitForConnected(10000)) {
@@ -244,6 +229,7 @@ void tst_QUdpSocket::initTestCase_data()
             newTestServer = false;
         socket.disconnectFromHost();
     }
+#endif
 
     QTest::addColumn<bool>("setProxy");
     QTest::addColumn<int>("proxyType");
@@ -257,8 +243,13 @@ void tst_QUdpSocket::initTestCase_data()
 
 void tst_QUdpSocket::initTestCase()
 {
+#ifdef QT_TEST_SERVER
+     QVERIFY(QtNetworkSettings::verifyConnection(QtNetworkSettings::socksProxyServerName(), 1080));
+     QVERIFY(QtNetworkSettings::verifyConnection(QtNetworkSettings::echoServerName(), 7));
+#else
     if (!QtNetworkSettings::verifyTestNetworkSettings())
         QSKIP("No network test server available");
+#endif
     allAddresses = QNetworkInterface::allAddresses();
     m_skipUnsupportedIPv6Tests = shouldSkipIpv6TestsForBrokenSetsockopt();
 
@@ -289,7 +280,7 @@ void tst_QUdpSocket::initTestCase()
     qDebug() << "Will use multicast groups" << multicastGroup4 << multicastGroup6 << linklocalMulticastGroups;
 
     m_workaroundLinuxKernelBug = shouldWorkaroundLinuxKernelBug();
-    if (EmulationDetector::isRunningArmOnX86())
+    if (QTestPrivate::isRunningArmOnX86())
         QSKIP("This test is unreliable due to QEMU emulation shortcomings.");
 }
 
@@ -300,7 +291,7 @@ void tst_QUdpSocket::init()
 #if QT_CONFIG(socks5)
         QFETCH_GLOBAL(int, proxyType);
         if (proxyType == QNetworkProxy::Socks5Proxy) {
-            QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::Socks5Proxy, QtNetworkSettings::serverName(), 1080));
+            QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::Socks5Proxy, QtNetworkSettings::socksProxyServerName(), 1080));
         }
 #else
         QSKIP("No proxy support");
@@ -429,7 +420,6 @@ void tst_QUdpSocket::broadcasting()
                 QEXPECT_FAIL("",
                              "Broadcasting to 255.255.255.255 does not work on FreeBSD",
                              Abort);
-                QVERIFY(false); // seems that QFAIL() doesn't respect the QEXPECT_FAIL() :/
 #endif
                 QFAIL("Network operation timed out");
             }
@@ -875,7 +865,7 @@ void tst_QUdpSocket::writeDatagram()
     qRegisterMetaType<QAbstractSocket::SocketError>("QAbstractSocket::SocketError");
 
     for(int i=0;;i++) {
-        QSignalSpy errorspy(&client, SIGNAL(error(QAbstractSocket::SocketError)));
+        QSignalSpy errorspy(&client, SIGNAL(errorOccurred(QAbstractSocket::SocketError)));
         QSignalSpy bytesspy(&client, SIGNAL(bytesWritten(qint64)));
 
         qint64 written = client.writeDatagram(QByteArray(i * 1024, 'w'), serverAddress,
@@ -915,7 +905,7 @@ void tst_QUdpSocket::performance()
     client.connectToHost(serverAddress, server.localPort());
     QVERIFY(client.waitForConnected(10000));
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     qint64 nbytes = 0;
@@ -962,8 +952,10 @@ void tst_QUdpSocket::bindMode()
 
     // Depending on the user's privileges, this or will succeed or
     // fail. Admins are allowed to reuse the address, but nobody else.
-    if (!socket2.bind(socket.localPort(), QUdpSocket::ReuseAddressHint), socket2.errorString().toLatin1().constData())
-        qWarning("Failed to bind with QUdpSocket::ReuseAddressHint, user isn't an administrator?");
+    if (!socket2.bind(socket.localPort(), QUdpSocket::ReuseAddressHint)) {
+        qWarning("Failed to bind with QUdpSocket::ReuseAddressHint(%s), user isn't an administrator?",
+                 qPrintable(socket2.errorString()));
+    }
     socket.close();
     QVERIFY2(socket.bind(0, QUdpSocket::ShareAddress), socket.errorString().toLatin1().constData());
     QVERIFY(!socket2.bind(socket.localPort()));
@@ -979,7 +971,7 @@ void tst_QUdpSocket::writeDatagramToNonExistingPeer_data()
     QTest::addColumn<bool>("bind");
     QTest::addColumn<QHostAddress>("peerAddress");
     QHostAddress localhost(QHostAddress::LocalHost);
-    QList<QHostAddress> serverAddresses(QHostInfo::fromName(QtNetworkSettings::serverName()).addresses());
+    QList<QHostAddress> serverAddresses(QHostInfo::fromName(QtNetworkSettings::socksProxyServerName()).addresses());
     if (serverAddresses.isEmpty())
         return;
 
@@ -993,7 +985,7 @@ void tst_QUdpSocket::writeDatagramToNonExistingPeer_data()
 
 void tst_QUdpSocket::writeDatagramToNonExistingPeer()
 {
-    if (QHostInfo::fromName(QtNetworkSettings::serverName()).addresses().isEmpty())
+    if (QHostInfo::fromName(QtNetworkSettings::socksProxyServerName()).addresses().isEmpty())
         QFAIL("Could not find test server address");
     QFETCH(bool, bind);
     QFETCH(QHostAddress, peerAddress);
@@ -1013,7 +1005,7 @@ void tst_QUdpSocket::writeToNonExistingPeer_data()
 {
     QTest::addColumn<QHostAddress>("peerAddress");
     QHostAddress localhost(QHostAddress::LocalHost);
-    QList<QHostAddress> serverAddresses(QHostInfo::fromName(QtNetworkSettings::serverName()).addresses());
+    QList<QHostAddress> serverAddresses(QHostInfo::fromName(QtNetworkSettings::socksProxyServerName()).addresses());
     if (serverAddresses.isEmpty())
         return;
 
@@ -1026,7 +1018,7 @@ void tst_QUdpSocket::writeToNonExistingPeer_data()
 void tst_QUdpSocket::writeToNonExistingPeer()
 {
     QSKIP("Connected-mode UDP sockets and their behaviour are erratic");
-    if (QHostInfo::fromName(QtNetworkSettings::serverName()).addresses().isEmpty())
+    if (QHostInfo::fromName(QtNetworkSettings::socksProxyServerName()).addresses().isEmpty())
         QFAIL("Could not find test server address");
     QFETCH(QHostAddress, peerAddress);
     quint16 peerPort = 34534;
@@ -1034,7 +1026,7 @@ void tst_QUdpSocket::writeToNonExistingPeer()
 
     QUdpSocket sConnected;
     QSignalSpy sConnectedReadyReadSpy(&sConnected, SIGNAL(readyRead()));
-    QSignalSpy sConnectedErrorSpy(&sConnected, SIGNAL(error(QAbstractSocket::SocketError)));
+    QSignalSpy sConnectedErrorSpy(&sConnected, SIGNAL(errorOccurred(QAbstractSocket::SocketError)));
     sConnected.connectToHost(peerAddress, peerPort, QIODevice::ReadWrite);
     QVERIFY(sConnected.waitForConnected(10000));
 
@@ -1078,14 +1070,23 @@ void tst_QUdpSocket::outOfProcessConnectedClientServerTest()
     QSKIP("No qprocess support", SkipAll);
 #else
     QProcess serverProcess;
-    serverProcess.start(QLatin1String("clientserver/clientserver server 1 1"),
+    serverProcess.start(QLatin1String("clientserver/clientserver server 1 1"), {},
                         QIODevice::ReadWrite | QIODevice::Text);
-    QVERIFY2(serverProcess.waitForStarted(3000),
-             qPrintable("Failed to start subprocess: " + serverProcess.errorString()));
+
+    const auto serverProcessCleaner = qScopeGuard([&serverProcess] {
+        serverProcess.kill();
+        serverProcess.waitForFinished();
+    });
+
+    if (!serverProcess.waitForStarted(3000))
+        QSKIP("Failed to start server as a subprocess");
 
     // Wait until the server has started and reports success.
-    while (!serverProcess.canReadLine())
-        QVERIFY(serverProcess.waitForReadyRead(3000));
+    while (!serverProcess.canReadLine()) {
+       if (!serverProcess.waitForReadyRead(3000))
+           QSKIP("No output from the server process, bailing out");
+    }
+
     QByteArray serverGreeting = serverProcess.readLine();
     QVERIFY(serverGreeting != QByteArray("XXX\n"));
     int serverPort = serverGreeting.trimmed().toInt();
@@ -1093,14 +1094,23 @@ void tst_QUdpSocket::outOfProcessConnectedClientServerTest()
 
     QProcess clientProcess;
     clientProcess.start(QString::fromLatin1("clientserver/clientserver connectedclient %1 %2")
-                        .arg(QLatin1String("127.0.0.1")).arg(serverPort),
+                        .arg(QLatin1String("127.0.0.1")).arg(serverPort), {},
                         QIODevice::ReadWrite | QIODevice::Text);
-    QVERIFY2(clientProcess.waitForStarted(3000),
-             qPrintable("Failed to start subprocess: " + clientProcess.errorString()));
 
-    // Wait until the server has started and reports success.
-    while (!clientProcess.canReadLine())
-        QVERIFY(clientProcess.waitForReadyRead(3000));
+    const auto clientProcessCleaner = qScopeGuard([&clientProcess] {
+        clientProcess.kill();
+        clientProcess.waitForFinished();
+    });
+
+    if (!clientProcess.waitForStarted(3000))
+        QSKIP("Client process did not start");
+
+    // Wait until the client has started and reports success.
+    while (!clientProcess.canReadLine()) {
+        if (!clientProcess.waitForReadyRead(3000))
+            QSKIP("No output from the client process, bailing out");
+    }
+
     QByteArray clientGreeting = clientProcess.readLine();
     QCOMPARE(clientGreeting, QByteArray("ok\n"));
 
@@ -1125,11 +1135,6 @@ void tst_QUdpSocket::outOfProcessConnectedClientServerTest()
         QCOMPARE(serverData.at(i * 3 + 2).trimmed().mid(8).toInt(),
                  sdata.mid(4).trimmed().toInt() * 2);
     }
-
-    clientProcess.kill();
-    QVERIFY(clientProcess.waitForFinished());
-    serverProcess.kill();
-    QVERIFY(serverProcess.waitForFinished());
 #endif
 }
 
@@ -1139,14 +1144,23 @@ void tst_QUdpSocket::outOfProcessUnconnectedClientServerTest()
     QSKIP("No qprocess support", SkipAll);
 #else
     QProcess serverProcess;
-    serverProcess.start(QLatin1String("clientserver/clientserver server 1 1"),
+    serverProcess.start(QLatin1String("clientserver/clientserver server 1 1"), {},
                         QIODevice::ReadWrite | QIODevice::Text);
-    QVERIFY2(serverProcess.waitForStarted(3000),
-             qPrintable("Failed to start subprocess: " + serverProcess.errorString()));
+
+    const auto serverProcessCleaner = qScopeGuard([&serverProcess] {
+        serverProcess.kill();
+        serverProcess.waitForFinished();
+    });
+
+    if (!serverProcess.waitForStarted(3000))
+        QSKIP("Failed to start the server subprocess");
 
     // Wait until the server has started and reports success.
-    while (!serverProcess.canReadLine())
-        QVERIFY(serverProcess.waitForReadyRead(3000));
+    while (!serverProcess.canReadLine()) {
+        if (!serverProcess.waitForReadyRead(3000))
+            QSKIP("No output from the server, probably, it is not running");
+    }
+
     QByteArray serverGreeting = serverProcess.readLine();
     QVERIFY(serverGreeting != QByteArray("XXX\n"));
     int serverPort = serverGreeting.trimmed().toInt();
@@ -1154,14 +1168,23 @@ void tst_QUdpSocket::outOfProcessUnconnectedClientServerTest()
 
     QProcess clientProcess;
     clientProcess.start(QString::fromLatin1("clientserver/clientserver unconnectedclient %1 %2")
-                        .arg(QLatin1String("127.0.0.1")).arg(serverPort),
+                        .arg(QLatin1String("127.0.0.1")).arg(serverPort), {},
                         QIODevice::ReadWrite | QIODevice::Text);
-    QVERIFY2(clientProcess.waitForStarted(3000),
-             qPrintable("Failed to start subprocess: " + clientProcess.errorString()));
 
-    // Wait until the server has started and reports success.
-    while (!clientProcess.canReadLine())
-        QVERIFY(clientProcess.waitForReadyRead(3000));
+    const auto clientProcessCleaner = qScopeGuard([&clientProcess] {
+        clientProcess.kill();
+        clientProcess.waitForFinished();
+    });
+
+    if (!clientProcess.waitForStarted(3000))
+        QSKIP("Failed to start the client's subprocess");
+
+    // Wait until the client has started and reports success.
+    while (!clientProcess.canReadLine()) {
+        if (!clientProcess.waitForReadyRead(3000))
+            QSKIP("The client subprocess produced not output, exiting.");
+    }
+
     QByteArray clientGreeting = clientProcess.readLine();
     QCOMPARE(clientGreeting, QByteArray("ok\n"));
 
@@ -1187,11 +1210,6 @@ void tst_QUdpSocket::outOfProcessUnconnectedClientServerTest()
         QCOMPARE(serverData.at(i * 3 + 2).trimmed().mid(8).toInt(),
                  sdata.mid(4).trimmed().toInt() * 2);
     }
-
-    clientProcess.kill();
-    QVERIFY(clientProcess.waitForFinished());
-    serverProcess.kill();
-    QVERIFY(serverProcess.waitForFinished());
 #endif
 }
 
@@ -1240,9 +1258,6 @@ void tst_QUdpSocket::multicastTtlOption_data()
 
 void tst_QUdpSocket::multicastTtlOption()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH_GLOBAL(bool, setProxy);
     QFETCH(QHostAddress, bindAddress);
     QFETCH(int, ttl);
@@ -1290,9 +1305,6 @@ void tst_QUdpSocket::multicastLoopbackOption_data()
 
 void tst_QUdpSocket::multicastLoopbackOption()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH_GLOBAL(bool, setProxy);
     QFETCH(QHostAddress, bindAddress);
     QFETCH(int, loopback);
@@ -1329,9 +1341,6 @@ void tst_QUdpSocket::multicastJoinBeforeBind_data()
 
 void tst_QUdpSocket::multicastJoinBeforeBind()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH(QHostAddress, groupAddress);
 
     QUdpSocket udpSocket;
@@ -1351,9 +1360,6 @@ void tst_QUdpSocket::multicastLeaveAfterClose_data()
 
 void tst_QUdpSocket::multicastLeaveAfterClose()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH_GLOBAL(bool, setProxy);
     QFETCH(QHostAddress, groupAddress);
     if (setProxy)
@@ -1398,9 +1404,6 @@ void tst_QUdpSocket::setMulticastInterface_data()
 
 void tst_QUdpSocket::setMulticastInterface()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH_GLOBAL(bool, setProxy);
     QFETCH(QNetworkInterface, iface);
     QFETCH(QHostAddress, address);
@@ -1458,9 +1461,6 @@ void tst_QUdpSocket::multicast_data()
 
 void tst_QUdpSocket::multicast()
 {
-#ifdef Q_OS_WINRT
-    QSKIP("WinRT does not support multicast.");
-#endif
     QFETCH_GLOBAL(bool, setProxy);
     QFETCH(QHostAddress, bindAddress);
     QFETCH(bool, bindResult);
@@ -1549,7 +1549,7 @@ void tst_QUdpSocket::echo_data()
 void tst_QUdpSocket::echo()
 {
     QFETCH(bool, connect);
-    QHostInfo info = QHostInfo::fromName(QtNetworkSettings::serverName());
+    QHostInfo info = QHostInfo::fromName(QtNetworkSettings::echoServerName());
     QVERIFY(info.addresses().count());
     QHostAddress remote = info.addresses().first();
 
@@ -1638,15 +1638,14 @@ void tst_QUdpSocket::linkLocalIPv6()
         sockets << s;
     }
 
-    QUdpSocket neutral;
-    QVERIFY(neutral.bind(QHostAddress(QHostAddress::AnyIPv6)));
-    QSignalSpy neutralReadSpy(&neutral, SIGNAL(readyRead()));
-
     QByteArray testData("hello");
     foreach (QUdpSocket *s, sockets) {
+        QUdpSocket neutral;
+        QVERIFY(neutral.bind(QHostAddress(QHostAddress::AnyIPv6)));
+        QSignalSpy neutralReadSpy(&neutral, SIGNAL(readyRead()));
+
         QSignalSpy spy(s, SIGNAL(readyRead()));
 
-        neutralReadSpy.clear();
         QVERIFY(s->writeDatagram(testData, s->localAddress(), neutral.localPort()));
         QTRY_VERIFY(neutralReadSpy.count() > 0); //note may need to accept a firewall prompt
 
